/*
 * yfreertos.hpp
 *
 *  Created on: Dec 25, 2020
 *      Author: yrs
 */

#ifndef INC_YFREERTOS_H_
#define INC_YFREERTOS_H_

namespace YFreeRTOS {
inline static void callbackWithTaskNotify(osThreadId_t &task) {
	BaseType_t xHigherPriorityTaskWoken = false;
	vTaskNotifyGiveFromISR(static_cast<TaskHandle_t>(task),
			&xHigherPriorityTaskWoken);
	portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
}
}

// 双缓冲区，相对于使用堆来讲性能极高，但是要求数据切换速度在允许范围之内。
template<class T>
struct DoubleDataBuffer {
public:
	DataBuffer *get() {
		return rest ? &buffers[index] : nullptr;
	}
	// 切换双缓冲区
	void swap() {
		taskENTER_CRITICAL();
		index = 1 - index;
		rest--;
		taskEXIT_CRITICAL();
	}
	void release() {
		taskENTER_CRITICAL();
		rest++;
		taskEXIT_CRITICAL();
	}
private:
	std::array<T, 2> buffers;
	uint8_t index = 0;
	uint8_t rest = 2;
};

#endif // INC_YFREERTOS_H_
